@@ -1,6 +1,9 @@
 use std::process::{Command, Stdio};
 use std::path::{PathBuf, Path};
+use self::super::apply_macros;
 use std::borrow::Cow;
+use std::ffi::OsStr;
+use memchr::memmem;
 use std::{env, fs};
 
 
@@ -19,8 +22,8 @@ impl ResourceCompiler {
         self.compiler.is_some()
     }
 
-    pub fn compile_resource(&self, out_dir: &str, prefix: &str, resource: &str) -> String {
-        self.compiler.as_ref().expect("Not supported but we got to compile_resource()?").compile(out_dir, prefix, resource)
+    pub fn compile_resource<Ms: AsRef<OsStr>, Mi: IntoIterator<Item = Ms>>(&self, out_dir: &str, prefix: &str, resource: &str, macros: Mi) -> String {
+        self.compiler.as_ref().expect("Not supported but we got to compile_resource()?").compile(out_dir, prefix, resource, macros)
     }
 }
 
@@ -30,7 +33,7 @@ enum CompilerType {
     /// LLVM-RC
     ///
     /// Requires a separate C preprocessor step on the source RC file
-    LlvmRc,
+    LlvmRc { has_no_preprocess: bool, },
     /// MinGW windres
     WindRes,
 }
@@ -52,7 +55,7 @@ impl Compiler {
             return Some(guess_compiler_variant(&rc));
         }
 
-        if target.ends_with("-pc-windows-gnu") {
+        if target.ends_with("-pc-windows-gnu") || target.ends_with("-pc-windows-gnullvm") {
             let executable = format!("{}-w64-mingw32-windres", &target[0..target.find('-').unwrap_or_default()]);
             if is_runnable(&executable) {
                 return Some(Compiler {
@@ -63,7 +66,14 @@ impl Compiler {
         } else if target.ends_with("-pc-windows-msvc") {
             if is_runnable("llvm-rc") {
                 return Some(Compiler {
-                    tp: CompilerType::LlvmRc,
+                    tp: CompilerType::LlvmRc {
+                        has_no_preprocess: Command::new("llvm-rc")
+                            .arg("/?")
+                            .output()
+                            .ok()
+                            .map(|out| memmem::find(&out.stdout, b"no-preprocess").is_some())
+                            .unwrap_or(false),
+                    },
                     executable: "llvm-rc".into(),
                 });
             }
@@ -72,16 +82,13 @@ impl Compiler {
         None
     }
 
-    pub fn compile(&self, out_dir: &str, prefix: &str, resource: &str) -> String {
+    pub fn compile<Ms: AsRef<OsStr>, Mi: IntoIterator<Item = Ms>>(&self, out_dir: &str, prefix: &str, resource: &str, macros: Mi) -> String {
+        let out_file = format!("{}/{}.lib", out_dir, prefix);
         match self.tp {
-            CompilerType::LlvmRc => {
-                let out_file = format!("{}/{}.lib", out_dir, prefix);
-
+            CompilerType::LlvmRc { has_no_preprocess } => {
                 let preprocessed_path = format!("{}/{}-preprocessed.rc", out_dir, prefix);
                 fs::write(&preprocessed_path,
-                          cc::Build::new()
-                              .define("RC_INVOKED", None)
-                              .flag("-xc")
+                          cc_xc(apply_macros_cc(cc::Build::new().define("RC_INVOKED", None), macros))
                               .file(resource)
                               .cargo_metadata(false)
                               .include(out_dir)
@@ -89,29 +96,54 @@ impl Compiler {
                     .unwrap();
 
                 try_command(Command::new(&self.executable[..])
-                                .args(&["/fo", &out_file, "--", &preprocessed_path])
+                                .args(&["/fo", &out_file])
+                                .args(if has_no_preprocess {
+                                    // We already preprocessed using CC. llvm-rc preprocessing
+                                    // requires having clang in PATH, which more exotic toolchains
+                                    // may not necessarily have.
+                                    &["/no-preprocess"][..]
+                                } else {
+                                    &[][..]
+                                })
+                                .args(&["--", &preprocessed_path])
                                 .stdin(Stdio::piped())
                                 .current_dir(or_curdir(Path::new(resource).parent().expect("Resource parent nonexistent?"))),
                             Path::new(&self.executable[..]),
                             "compile",
                             &preprocessed_path,
                             &out_file);
-                out_file
             }
             CompilerType::WindRes => {
-                let out_file = format!("{}/lib{}.a", out_dir, prefix);
-                try_command(Command::new(&self.executable[..])
-                                .args(&["--input", resource, "--output-format=coff", "--output", &out_file, "--include-dir", out_dir]),
+                try_command(apply_macros(Command::new(&self.executable[..])
+                                             .args(&["--input", resource, "--output-format=coff", "--output", &out_file, "--include-dir", out_dir]),
+                                         "-D",
+                                         macros),
                             Path::new(&self.executable[..]),
                             "compile",
                             resource,
                             &out_file);
-                out_file
             }
         }
+        out_file
     }
 }
 
+fn apply_macros_cc<'t, Ms: AsRef<OsStr>, Mi: IntoIterator<Item = Ms>>(to: &'t mut cc::Build, macros: Mi) -> &'t mut cc::Build {
+    for m in macros {
+        let mut m = m.as_ref().to_str().expect("macros must be UTF-8 in this configuration").splitn(2, '=');
+        to.define(m.next().unwrap(), m.next());
+    }
+    to
+}
+
+fn cc_xc(to: &mut cc::Build) -> &mut cc::Build {
+    if to.get_compiler().is_like_msvc() {  // clang-cl
+        to.flag("-Xclang");
+    }
+    to.flag("-xc");
+    to
+}
+
 fn try_command(cmd: &mut Command, exec: &Path, action: &str, whom: &str, whre: &str) {
     match cmd.status() {
         Ok(stat) if stat.success() => {}
@@ -139,10 +171,10 @@ fn guess_compiler_variant(s: &str) -> Compiler {
                     executable: s.to_string().into(),
                     tp: CompilerType::WindRes,
                 }
-            } else if out.stdout.starts_with(b"OVERVIEW: Resource Converter") {
+            } else if out.stdout.starts_with(b"OVERVIEW: Resource Converter") || out.stdout.starts_with(b"OVERVIEW: LLVM Resource Converter") {
                 Compiler {
                     executable: s.to_string().into(),
-                    tp: CompilerType::LlvmRc,
+                    tp: CompilerType::LlvmRc { has_no_preprocess: memmem::find(&out.stdout, b"no-preprocess").is_some() },
                 }
             } else {
                 panic!("Unknown RC compiler variant: {}", s)
